local _ARTG = _Avyr_Raid_Tools_ADDON_Globals
local Widgets = _Avyr_Global_ADDON_Elements_Lib:Initialize("AvyrRaidTools","Widgets")
local Loc = _Avyr_Global_ADDON_Elements_Lib:Initialize("AvyrRaidTools","Localisation")
local BorderSize, Padding = Widgets.BorderSize, Widgets.Padding;
local insert,wipe = table.insert,table.wipe

_ARTG.DataFuncs = {}

local Data = _ARTG.DataFuncs

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------General functions-----------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Data.cloneTable = function(table)
	if type(table) == "table" then 
		local clone = {}
		
		for i,v in pairs(table) do 
			if type(v) == "table" then 
				clone[i] = Data.cloneTable(v)
			else 
				clone[i] = v
			end 
		end 

		return clone
	else 
		return table
	end 
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------Expansions, Raid and Boss Data-----------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local DefaultBossImage = "Interface\\EncounterJournal\\UI-EJ-BOSS-Default"
local expansionsData = {
	--Names will be overwritten with localised names 
	[1] = {
		Name = "Classic",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-AQRUINS",0,1,0.3,0.7},
	},
	[2] = {
		Name = "TBC",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-HELLFIRECITADEL",0,1,0.2,0.6},
	},
	[3] = {
		Name = "WotLK",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-ICECROWNCITADEL",0,1,0.1,0.5},
	},
	[4] = {
		Name = "Cata",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-FallofDeathwing",0,1,0.3,0.7},
	},
	[5] = {
		Name = "Panda",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-TempleoftheJadeSerpent",0,1,0.2,0.6},
	},
	[6] = {
		Name = "WoD",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Draenor",0,1,0.3,0.7},
	},
	[7] = {
		Name = "Legion",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Antorus",0,1,0.3,0.7},
	},
	[8] = {
		Name = "BfA",
		Image = {"Interface\\LFGFRAME\\UI-LFG-BACKGROUND-Ataldazar",0,1,0.3,0.7},
	},
}
local raidsByExpansion = {}
local encounterInfoByID = {}
local bossByRaids = {}
local bossTemplates = {}
local bossMobIDLoadInfo = {
	--darmac (debugging)
	["76865"] = {1694,457,6},
	
	
	--wrathion
	["156818"] = {2329,1180,8},
	--maut
	["156523"] = {2327,1180,8},
	--The Prophet Skitra
	["157238"] = {2334,1180,8},
	--Dark Inquisitor Xanesh
	["156575"] = {2328,1180,8},
	--The Hivemind  
	["157253"] = {2333,1180,8},
	["157254"] = {2333,1180,8},
	--Shad'har the Insatiable
	["157231"] = {2335,1180,8},
	--Drest'agath
	["157602"] = {2343,1180,8},
	--Vexiona
	["157354"] = {2336,1180,8},
	--Ra-den the Despoiled
	["156866"] = {2331,1180,8},
	--Il'gynoth, Corruption Reborn
	["158328"] = {2345,1180,8},
	--Carapace of N'Zoth
	["157439"] = {2337,1180,8},
	--N'Zoth, the Corruptor
	["158041"] = {2344,1180,8},
}
local bossLocationInfo = {
	
}

Data.RequiresLoadPrompt = function(Action,...)
	local encounterToLoad = nil
	if Action == "GUID" then 
		local GUID = ...
		local _,_,_,_,_,mobID = strsplit("-",GUID)
		if bossMobIDLoadInfo[mobID] then 
			return _ARTG.NoteDisplay.CallLoadNotePopup("show.bossNotes",unpack(bossMobIDLoadInfo[mobID]))
		end
	elseif Action == "ZoneID" then 
		
	end 
end


local function scanExpansions(forceScan,expansionIndex)
	if type(expansionIndex) == "number" then 
		if expansionIndex > 0 and expansionIndex <= EJ_GetNumTiers() and expansionsData[expansionIndex] then
			if expansionsData[expansionIndex].Checked and not forceScan then 
				return expansionIndex
			else 
				local Name,Link = EJ_GetTierInfo(expansionIndex)
				if Name then 
					expansionsData[expansionIndex].Name = Name 
					return  expansionIndex
				end
			end
		end
	else
		for i,v in ipairs(expansionsData) do 
			if not v.Checked or forceScan then 
				local Name,Link = EJ_GetTierInfo(i)
				if Name then 
					v.Name = Name 
				end
			end 
		end 
	end
end

local function createExpansionRaidList(expansionIndex,targetRaid)
	EJ_SelectTier(expansionIndex)

	local foundIt = false

	raidsByExpansion[expansionIndex] = raidsByExpansion[expansionIndex] or {}
	table.wipe(raidsByExpansion[expansionIndex])

	for raid = 1,100 do 
		local instanceID, name, description, bgImage, buttonImage1, loreImage, buttonImage2, dungeonAreaMapID, link, shouldDisplayDifficulty = EJ_GetInstanceByIndex(raid,true)
		if instanceID then 
			insert(raidsByExpansion[expansionIndex],instanceID)
			if type(targetRaid) == "number" and targetRaid == instanceID then 
				foundIt = true 
			end
		else
			break
		end 					
	end 

	return foundIt
end

local function createBossList(raidID,targetBoss)
	EJ_SelectInstance(raidID)

	bossByRaids[raidID] = bossByRaids[raidID] or {}
	table.wipe(bossByRaids[raidID])

	local foundIt = false 
	for encounter = 1,100 do 
		local name, description, EJ_encounterID, rootSectionID, link, instanceID, encounterID = EJ_GetEncounterInfoByIndex(encounter,raidID)
		local _, _, _, _, bossImage = EJ_GetCreatureInfo(1, EJ_encounterID);
		
		if name and encounterID then 
			insert(bossByRaids[raidID],encounterID)
			encounterInfoByID[encounterID] = {EJ_encounterID = EJ_encounterID, imagePath = bossImage};
			if type(targetBoss) == "number" and encounterID == targetBoss then 
				foundIt = true 
			end
		else 
			if not name then --Kara (TBC) servant quarter boss catch
				break 
			end 
		end 
	end 
	
	return foundIt
end

local function scanExpansionsForRaid(raidID)
	local foundIt = false
	for i = 1,EJ_GetNumTiers() do 
		if raidsByExpansion[i] then 
			if tContains(raidsByExpansion[i],raidID) then 
				foundIt = true 
			end
		else
			if createExpansionRaidList(i,raidID) then 
				foundIt = true 
			end
		end 
		if foundIt then 
			return true 
		end 
	end
	return false 
end

local function ScanRaidsForBoss(bossID)
	for i = 1,EJ_GetNumTiers() do 
		if not raidsByExpansion[i] then 
			createExpansionRaidList(i,raidID)
		end 

		for _,raidID in ipairs(raidsByExpansion[i]) do 
			if createBossList(raidID,bossID) then 
				return true 
			end 
		end
	end
	return false 
end 

local function scanRaids(forceScan,raidID,expansionIndex) 
	if type(raidID) == "number" then 
		if bossByRaids[raidID] then --force scan integration
			return true 
		end

		local raidFound = false 
		if _ARTG.DataFuncs.GetRaidInfo("exp.find",expansionIndex) then 
			if not raidsByExpansion[expansionIndex] or forceScan then 
				raidFound = createExpansionRaidList(expansionIndex,raidID)
			end
	
			if raidFound then 
				return true 
			end
		end 

		return scanExpansionsForRaid(raidID)
	elseif not raidsByExpansion[expansionIndex] or forceScan then 
		createExpansionRaidList(expansionIndex)
	end 

	return false 
end

local function scanBosses(forceScan,bossID,raidID)
	if type(bossID) == "number" then 
		if encounterInfoByID[bossID] then 
			return true 
		end
		
		local bossFound = false
		if type(raidID) == "number" and EJ_GetInstanceInfo(raidID) then 
			 bossFound = createBossList(raidID,bossID)
		end 

		if bossFound then 
			return true 
		end

		return ScanRaidsForBoss(bossID)
	end
	return false 
end

Data.GetRaidInfo = function(Action,...)
	if Action == "none" then 

	elseif Action == "boss.find" then 
		local bossID,raidID = ...

		return scanBosses(false,bossID,raidID)	
	elseif Action == "boss.list" then 
		local raidID,expansionIndex = ...

		local foundIt = scanBosses(false)

		if type(raidID) == "number" then 
			createBossList(raidID)

			return bossByRaids[raidID]
		end

		return {}
	elseif Action:find("boss.data") then
		local bossID,raidID,expansionIndex = ...

		local name = "Unknown boss"
		local imagePath = DefaultBossImage

		if type(bossID) == "number" and scanBosses(false,bossID,raidID,expansionIndex) then 
			name = EJ_GetEncounterInfo(encounterInfoByID[bossID]["EJ_encounterID"])
			imagePath = encounterInfoByID[bossID]["imagePath"]
		end

		if Action:find(".name") then 
			return name
		elseif Action:find(".image") then 
			return imagePath
		else 
			return name, imagePath
		end
	elseif Action == "raid.find" then 
		local raidID,expansionIndex = ...

		return scanRaids(false,raidID,expansionIndex)
	elseif Action == "raid.list" then 
		local expansionIndex = ...
		
		if expansionIndex then 
			scanRaids(false,nil,expansionIndex)	
			if raidsByExpansion[expansionIndex] then 
				return raidsByExpansion[expansionIndex]
			else
				return bossByRaids
			end 
			return bossByRaids
		end

		return {}
	elseif Action:find("raid.data") then 
		local raidID = ...
		local name = "Unknown raid"
		local imagePath = DefaultBossImage

		if type(raidID) == "number" then 
			local RaidName,_,_,Icon = EJ_GetInstanceInfo(raidID) 
			if RaidName then 
				name = RaidName
			end 
			if Icon then 
				imagePath = Icon
			end
		end

		if Action:find(".name") then 
			return name
		elseif Action:find(".image") then 
			return imagePath
		else 
			return name,imagePath
		end 
	elseif Action == "exp.find" then 
		local expansionIndex = ...
		if type(expansionIndex) == "number" and expansionIndex > 0 and expansionIndex <= EJ_GetNumTiers() and expansionsData[expansionIndex] then 
			return true 
		else 
			return false 
		end 
	elseif Action == "exp.list" then
		scanExpansions()

		return expansionsData
	elseif Action:find("exp.data") then 
		--exp.data.image
		--exp.data.name
		local expansionIndex = ...
		local Index = nil

		Index = scanExpansions(false,expansionIndex)
		
		if Index then 
			if Action:find(".image") then 
				return expansionsData[Index].Image[1],expansionsData[Index].Image[2],expansionsData[Index].Image[3],expansionsData[Index].Image[4],expansionsData[Index].Image[5]
			else --".name"
				return expansionsData[Index].Name
			end 
		end
	end
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------Notes------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local selectedNote = {}
Data.NoteData = function(Action,...)
	if Action == "current.set" then 
		local NoteTable,expansionIndex,raidID,bossID = ...

		local ClonedNote = Data.cloneTable(NoteTable)
		ClonedNote.Expansion = expansionIndex
		ClonedNote.Raid = raidID
		ClonedNote.Boss = bossID
		_ARTG.SV.SaveVariable("CurrentNote",ClonedNote)
	elseif Action == "current.get" then 
		return _ARTG.SV.GetVariable("CurrentNote")
	elseif Action == "current.set.selected" then 
		_ARTG.SV.SaveVariable("CurrentNote",Data.cloneTable(selectedNote))
	elseif Action == "selected.set" then 
		local NoteTable,expansionIndex,raidID,bossID = ...
		
		local ClonedNote = Data.cloneTable(NoteTable)
		ClonedNote.Expansion = expansionIndex
		ClonedNote.Raid = raidID
		ClonedNote.Boss = bossID

		wipe(selectedNote)

		selectedNote = ClonedNote
	elseif Action == "selected.get" then 
		return Data.cloneTable(selectedNote)
	end
end

----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------Cooldowns Data-------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------Inspect Data--------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------






----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
---------------------------------------------------------------------------------------SOUND Data---------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
local soundTable = {
	{Loc:Translation("WORLD_QUEST_START_SOUND"),
	Loc:Translation("RAID_BOSS_DEFEATED_SOUND"),
	Loc:Translation("SCENARIO_ENDING_SOUND"),
	Loc:Translation("ALARM_CLOCK_1_SOUND"),
	Loc:Translation("ALARM_CLOCK_2_SOUND"),
	Loc:Translation("ALARM_CLOCK_3_SOUND"),
	Loc:Translation("RAID_WARNING_SOUND"),
	Loc:Translation("PING_SOUND"),
	Loc:Translation("MONEY_SOUND"),
	Loc:Translation("OPEN_BAGS_SOUND"),
	Loc:Translation("READY_CHECK_SOUND"),},
	{73275,50111,31754,18871,12867,12889,8959,3175,120,862,8960},
}

local ShareMediaLib = LibStub("LibSharedMedia-3.0")
local sharedMediaLibHashTable = ShareMediaLib:HashTable(ShareMediaLib.MediaType.SOUND)

ART_global_testFunc = function()
	local l = ShareMediaLib:HashTable(ShareMediaLib.MediaType.SOUND)


	for i,v in pairs(l) do 
		print(i,v)
	end
	
	print(ShareMediaLib:Fetch(ShareMediaLib.MediaType.SOUND,"None"))
end

--/run ART_global_testFunc()
Data.Sounds = function(Action,...)
	if Action == "sound.play" then 
		local sound = ...
		if type(sound) == "number" then 
			PlaySound(sound,"Master")
		elseif type(sound) == "string" and sharedMediaLibHashTable[sound] then 
			PlaySoundFile(sharedMediaLibHashTable[sound],"Master")
		end
	elseif Action == "list.get" then 
		local soundSelect = Data.cloneTable(soundTable)
		for i,v in pairs(sharedMediaLibHashTable) do 
			insert(soundSelect[1],i)
			insert(soundSelect[2],i)
		end
		return soundSelect
	elseif Action:find("sound.get") then 
		local soundIndex
		if Action:find(".byIndex") then 
			soundIndex,soundName = ...
			if soundName and sharedMediaLibHashTable[soundName] then 
				return soundName
			end
		elseif Action:find(".byName") then 
			local soundName = ...
			if soundName then 
				if sharedMediaLibHashTable[soundName] then 
					return soundName
				end
				for i,v in pairs(soundTable[1]) do 
					if soundTable[1][i] == soundName then 
						soundIndex = i
					end 
				end
			end 	
		elseif Action:find(".byId") then 
			local soundId = ...
			if soundId then 
				if type(soundId) == "number" then
					for i,v in pairs(soundTable[1]) do 
						if soundTable[2][i] == soundId then 
							soundIndex = i
						end 
					end
				elseif type(soundId) == "string" and sharedMediaLibHashTable[soundId] then 
					return soundId
				end
			end 	
		end

		if soundIndex then 
			if type(soundIndex) == "number" and soundTable[1][soundIndex] then 
				if Action:find(".name") then 
					return soundTable[1][soundIndex]
				elseif Action:find(".id") then 
					return soundTable[2][soundIndex]
				end
			elseif type(soundIndex) == "string" and sharedMediaLibHashTable[soundIndex] then 
				return soundIndex --shared media id is the name
			end
		end

		return soundIndex
	end 
end


